@@ -176,26 +176,31 @@ def rm_container(name):
         if cmd_stderr:
             log.debug(cmd_stderr)
 
+    def run_cmd(rm_cli_cmd):
+        subproc = subprocess.Popen(rm_cli_cmd,
+                                   stdout=subprocess.PIPE,
+                                   stderr=subprocess.PIPE,
+                                   universal_newlines=True)
+        cmd_stdout, cmd_stderr = subproc.communicate()
+        if cmd_stdout:
+            log.debug(cmd_stdout)
+        if cmd_stderr and \
+               cmd_stderr != 'Error response from daemon: ' \
+               'No such container: {}\n'.format(name):
+            log.debug(cmd_stderr)
+
     log.info('Removing container: %s' % name)
     rm_cli_cmd = [cli_cmd, 'rm']
-    # --storage is used as a mitigation of
+    rm_cli_cmd.append(name)
+    run_cmd(rm_cli_cmd)
+
+    # rm --storage is used as a mitigation of
     # https://github.com/containers/libpod/issues/3906
     # Also look https://bugzilla.redhat.com/show_bug.cgi?id=1747885
     if container_cli == 'podman':
-        rm_cli_cmd.extend(['--storage'])
-    rm_cli_cmd.append(name)
-    subproc = subprocess.Popen(rm_cli_cmd,
-                               stdout=subprocess.PIPE,
-                               stderr=subprocess.PIPE,
-                               universal_newlines=True)
-    cmd_stdout, cmd_stderr = subproc.communicate()
-    if cmd_stdout:
-        log.debug(cmd_stdout)
-    if cmd_stderr and \
-           cmd_stderr != 'Error response from daemon: ' \
-           'No such container: {}\n'.format(name):
-        log.debug(cmd_stderr)
-
+        rm_storage_cli_cmd = [cli_cmd, 'rm', '--storage']
+        rm_storage_cli_cmd.append(name)
+        run_cmd(rm_storage_cli_cmd)
 
 process_count = int(os.environ.get('PROCESS_COUNT',
                                    multiprocessing.cpu_count()))
@@ -229,6 +234,7 @@ for service in (json_data or []):
             service.get('config_image'),
             service.get('volumes', []),
             service.get('privileged', False),
+            service.get('keep_container', False),
         ]
 
     config_volume = service[0] or ''
@@ -237,6 +243,7 @@ for service in (json_data or []):
     config_image = service[3] or ''
     volumes = service[4] if len(service) > 4 else []
     privileged = service[5] if len(service) > 5 else False
+    keep_container = service[6] if len(service) > 6 else False
 
     if not manifest or not config_image:
         continue
@@ -247,6 +254,7 @@ for service in (json_data or []):
     log.debug('config_image %s' % config_image)
     log.debug('volumes %s' % volumes)
     log.debug('privileged %s' % privileged)
+    log.debug('keep_container %s' % keep_container)
     # We key off of config volume for all configs.
     if config_volume in configs:
         # Append puppet tags and manifest.
@@ -273,147 +281,9 @@ for service in (json_data or []):
 
 log.info('Service compilation completed.')
 
-if not os.path.exists(sh_script):
-    with open(sh_script, 'w') as script_file:
-        os.chmod(script_file.name, 0o755)
-        script_file.write("""#!/bin/bash
-        set -e
-        [ "$DEBUG" = "false" ] || set -x
-        mkdir -p /etc/puppet
-        cp -dR /tmp/puppet-etc/* /etc/puppet
-        rm -Rf /etc/puppet/ssl # not in use and causes permission errors
-        echo "{\\"step\\": $STEP}" > /etc/puppet/hieradata/docker_puppet.json
-        TAGS=""
-        if [ -n "$PUPPET_TAGS" ]; then
-            TAGS="--tags \"$PUPPET_TAGS\""
-        fi
-
-        CHECK_MODE=""
-        if [ -d "/tmp/puppet-check-mode" ]; then
-            mkdir -p /etc/puppet/check-mode
-            cp -a /tmp/puppet-check-mode/* /etc/puppet/check-mode
-            CHECK_MODE="--hiera_config /etc/puppet/check-mode/hiera.yaml"
-        fi
-
-        # Create a reference timestamp to easily find all files touched by
-        # puppet. The sync ensures we get all the files we want due to
-        # different timestamp.
-        origin_of_time=/var/lib/config-data/${NAME}.origin_of_time
-        touch $origin_of_time
-        sync
-
-        export NET_HOST="${NET_HOST:-false}"
-        set +e
-        if [ "$NET_HOST" == "false" ]; then
-            export FACTER_hostname=$HOSTNAME
-        fi
-        # $::deployment_type in puppet-tripleo
-        export FACTER_deployment_type=containers
-        export FACTER_uuid=$(cat /sys/class/dmi/id/product_uuid | tr '[:upper:]' '[:lower:]')
-        echo 'Running puppet'
-        # FIXME(bogdando): stdout may be falling behind of the logged syslog messages
-        set -x
-        /usr/bin/puppet apply --summarize \
-                --detailed-exitcodes \
-                --color=false \
-                --modulepath=/etc/puppet/modules:/usr/share/openstack-puppet/modules \
-                $TAGS \
-                $CHECK_MODE \
-                /etc/config.pp \
-                2>&1 | logger -s -t puppet-user
-        rc=${PIPESTATUS[0]}
-        [ "$DEBUG" = "false" ] && set +x
-        set -e
-        if [ $rc -ne 2 -a $rc -ne 0 ]; then
-            exit $rc
-        fi
-
-        verbosity=""
-        [ "$DEBUG" = "false" ] || verbosity="-v"
-
-        # Disables archiving
-        if [ -z "$NO_ARCHIVE" ]; then
-            archivedirs=("/etc" "/root" "/opt" "/var/lib/ironic/tftpboot" "/var/lib/ironic/httpboot" "/var/www" "/var/spool/cron" "/var/lib/nova/.ssh")
-            rsync_srcs=""
-            for d in "${archivedirs[@]}"; do
-                if [ -d "$d" ]; then
-                    rsync_srcs+=" $d"
-                fi
-            done
-            # On stack update, if a password was changed in a config file,
-            # some services (e.g. mysql) must change their internal state
-            # (e.g. password in mysql DB) when paunch restarts them; and
-            # they need the old password to achieve that.
-            # For those services, we update the config hash to notify
-            # paunch that a restart is needed, but we do not update the
-            # password file in container-puppet if the file already existed
-            # before and let the service regenerate it instead.
-            password_files="/root/.my.cnf"
-
-            exclude_files=""
-            for p in $password_files; do
-                if [ -f "$p" -a -f "/var/lib/config-data/${NAME}$p" ]; then
-                    exclude_files+=" --exclude=$p"
-                fi
-            done
-
-            # Exclude read-only mounted directories/files which we do not want
-            # to copy or delete.
-            ro_files="/etc/puppetlabs/ /opt/puppetlabs/"
-            for ro in $ro_files; do
-                if [ -e "$ro" ]; then
-                    exclude_files+=" --exclude=$ro"
-                fi
-            done
-
-            echo "Rsyncing config files from ${rsync_srcs} into /var/lib/config-data/${NAME}"
-            rsync -a $verbosity -R --delay-updates --delete-after $exclude_files $rsync_srcs /var/lib/config-data/${NAME}
-
-
-            # Also make a copy of files modified during puppet run
-            # This is useful for debugging
-            echo "Gathering files modified after $(stat -c '%y' $origin_of_time)"
-            mkdir -p /var/lib/config-data/puppet-generated/${NAME}
-            exec 5>&1
-            exec 1>/tmp/files_from
-            find $rsync_srcs -newer $origin_of_time -not -path '/etc/puppet*' -print0
-            exec 1>&5
-            echo "Files modified during puppet run:"
-            cat /tmp/files_from | sort | xargs -0 printf "%s\n"
-            echo "Rsyncing the modified files into /var/lib/config-data/puppet-generated/${NAME}"
-            rsync -a $verbosity -R -0 --delay-updates --delete-after $exclude_files \
-                --files-from=/tmp/files_from / /var/lib/config-data/puppet-generated/${NAME}
-
-            # Write a checksum of the config-data dir, this is used as a
-            # salt to trigger container restart when the config changes
-            # note: while being excluded from the output, password files
-            # are still included in checksum computation
-            additional_checksum_files=""
-            excluded_original_passwords=""
-            for p in $password_files; do
-                if [ -f "$p" ]; then
-                    additional_checksum_files+=" $p"
-                    excluded_original_passwords+=" --exclude=/var/lib/config-data/*${p}"
-                fi
-            done
-            # We need to exclude the swift ring backups as those change over time and
-            # containers do not need to restart if they change
-            EXCLUDE=--exclude='*/etc/swift/backups/*'\ --exclude='*/etc/libvirt/passwd.db'\ ${excluded_original_passwords}
-
-            # We need to repipe the tar command through 'tar xO' to force text
-            # output because otherwise the sed command cannot work. The sed is
-            # needed because puppet puts timestamps as comments in cron and
-            # parsedfile resources, hence triggering a change at every redeploy
-            tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/${NAME} $additional_checksum_files | tar xO | \
-                    sed '/^#.*HEADER.*/d' | md5sum | awk '{print $1}' > /var/lib/config-data/${NAME}.md5sum
-            tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/puppet-generated/${NAME} $additional_checksum_files --mtime='1970-01-01' | tar xO \
-                    | sed '/^#.*HEADER.*/d' | md5sum | awk '{print $1}' > /var/lib/config-data/puppet-generated/${NAME}.md5sum
-        fi
-        """)
-
 
 def mp_puppet_config(*args):
-    (config_volume, puppet_tags, manifest, config_image, volumes, privileged, check_mode) = args[0]
+    (config_volume, puppet_tags, manifest, config_image, volumes, privileged, check_mode, keep_container) = args[0]
     log = get_logger()
     log.info('Starting configuration of %s using image %s' %
              (config_volume, config_image))
@@ -424,6 +294,7 @@ def mp_puppet_config(*args):
     log.debug('volumes %s' % volumes)
     log.debug('privileged %s' % privileged)
     log.debug('check_mode %s' % check_mode)
+    log.debug('keep_container %s' % keep_container)
 
     with tempfile.NamedTemporaryFile() as tmp_man:
         with open(tmp_man.name, 'w') as man_file:
@@ -459,8 +330,15 @@ def mp_puppet_config(*args):
                 '--volume', '/var/lib/container-puppet/puppetlabs/:/opt/puppetlabs/:ro',
                 # Syslog socket for puppet logs
                 '--volume', '/dev/log:/dev/log:rw']
+
+        # Remove container by default after the run
+        # This should mitigate the "ghost container" issue described here
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1747885
+        # https://bugs.launchpad.net/tripleo/+bug/1840691
+        if not keep_container:
+            common_dcmd.append('--rm')
         if privileged:
-            common_dcmd.extend('--privileged')
+            common_dcmd.append('--privileged')
 
         if container_cli == 'podman':
             log_path = os.path.join(container_log_stdout_path, uname)
@@ -507,12 +385,8 @@ def mp_puppet_config(*args):
         count = 0
         log.debug('Running %s command: %s' % (container_cli, ' '.join(common_dcmd)))
         while count < 3:
-            if count == 0:
-                cmd = common_dcmd
-            else:
-                cmd = [cli_cmd, 'start', '-a', uname]
             count += 1
-            subproc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+            subproc = subprocess.Popen(common_dcmd, stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE, env=env,
                                        universal_newlines=True)
             cmd_stdout, cmd_stderr = subproc.communicate()
@@ -521,14 +395,14 @@ def mp_puppet_config(*args):
             # and 2 for success and resource changes. Other numbers are failures
             if retval in [0, 2]:
                 if cmd_stdout:
-                    log.debug('%s run succeeded: %s' % (cmd, cmd_stdout))
+                    log.debug('%s run succeeded: %s' % (common_dcmd, cmd_stdout))
                 if cmd_stderr:
                     log.warning(cmd_stderr)
                 # only delete successful runs, for debugging
                 rm_container(uname)
                 break
             time.sleep(3)
-            log.error('%s run failed after %s attempt(s): %s' % (cmd,
+            log.error('%s run failed after %s attempt(s): %s' % (common_dcmd,
                                                                  cmd_stderr,
                                                                  count))
             log.warning('Retrying running container: %s' % config_volume)
@@ -559,6 +433,7 @@ for config_volume in configs:
     config_image = service[3] or ''
     volumes = service[4] if len(service) > 4 else []
     privileged = service[5] if len(service) > 5 else False
+    keep_container = service[6] if len(service) > 6 else False
 
     if puppet_tags:
         puppet_tags = "file,file_line,concat,augeas,cron,%s" % puppet_tags
@@ -566,7 +441,7 @@ for config_volume in configs:
         puppet_tags = "file,file_line,concat,augeas,cron"
 
     process_map.append([config_volume, puppet_tags, manifest, config_image,
-                        volumes, privileged, check_mode])
+                        volumes, privileged, check_mode, keep_container])
 
 for p in process_map:
     log.debug('- %s' % p)
@@ -604,9 +479,10 @@ for infile in infiles:
         config_hashes = filter(None, config_hashes)
         config_hash = '-'.join(config_hashes)
         if config_hash:
-            env = v.get('environment', [])
-            env.append("TRIPLEO_CONFIG_HASH=%s" % config_hash)
             log.debug("Updating config hash for %s, config_volume=%s hash=%s" % (k, config_volume, config_hash))
+            # When python 27 support is removed, we will be able to use z = {**x, **y} to merge the dicts.
+            v.get('environment', {}).update({'TRIPLEO_CONFIG_HASH': config_hash})
+            env = v.get('environment')
             infile_data[k]['environment'] = env
 
     outfile = os.path.join(os.path.dirname(infile), "hashed-" + os.path.basename(infile))
